@@ -15,6 +15,7 @@ from OpenGL.GL import (
     glGetUniformLocation,
     glUniform1f,
     glUniform1i,
+    glUniform2fv,
     glUniform3fv,
     glUniform4fv,
     glUniformMatrix4fv,
@@ -95,26 +96,32 @@ layout (points) in;
 layout (triangle_strip, max_vertices=4) out;
 
 out vec2 varTex;
+out vec3 particle_world_pos;
 
 void main() {
-    vec4 center = view * model * gl_in[0].gl_Position;
-    varTex = vec2( 0, 0 );
-    gl_Position = projection * (center + vec4(particle_size, particle_size, 0, 0));
+    vec4 world_pos = model * gl_in[0].gl_Position;
+    particle_world_pos = world_pos.xyz;
+    vec4 center = view * world_pos;
+    
+    // Create proper billboard quad
+    varTex = vec2(0, 0);
+    gl_Position = projection * (center + vec4(-particle_size, -particle_size, 0, 0));
     EmitVertex();
 
-    varTex = vec2( 1, 0  );
+    varTex = vec2(1, 0);
     gl_Position = projection * (center + vec4(particle_size, -particle_size, 0, 0));
     EmitVertex();
 
-    varTex = vec2( 0, 1 );
+    varTex = vec2(0, 1);
     gl_Position = projection * (center + vec4(-particle_size, particle_size, 0, 0));
     EmitVertex();
 
-    varTex = vec2( 1, 1 );
-    gl_Position = projection * (center + vec4(-particle_size, -particle_size, 0, 0));
+    varTex = vec2(1, 1);
+    gl_Position = projection * (center + vec4(particle_size, particle_size, 0, 0));
     EmitVertex();
+    
+    EndPrimitive();
 }
-
 """
 
 
@@ -126,15 +133,23 @@ uniform vec3 object_color;
 uniform float opacity;
 
 in vec2 varTex;
+in vec3 particle_world_pos;
 
 layout(location = 0) out vec4 outFragColor;
 
 void main()
 {
-    vec4 tex_color;
-    tex_color = texture( tex_unit, varTex );
-
-    outFragColor = tex_color * vec4(object_color, opacity);
+    vec4 tex_color = texture(tex_unit, varTex);
+    
+    // Simple circular particle shape with smooth edges
+    vec2 center = vec2(0.5, 0.5);
+    float dist_from_center = length(varTex - center);
+    float circle_alpha = 1.0 - smoothstep(0.3, 0.5, dist_from_center);
+    
+    vec3 final_color = tex_color.rgb * object_color;
+    float final_alpha = tex_color.a * opacity * circle_alpha;
+    
+    outFragColor = vec4(final_color, final_alpha);
 }
 """
 
@@ -148,18 +163,30 @@ layout ( location = 3 ) in vec3 colors; // optional
 
 out vec2 tex_coords;
 out vec3 frag_color;
+out vec3 l_fragpos;
+out vec3 l_normal;
 
 uniform mat4 model;
 uniform mat4 view;
 uniform mat4 projection;
 uniform int view_mode;
-
-out vec3 l_fragpos;
-out vec3 l_normal;
+uniform float time;
 
 void main()
 {
-    gl_Position = vec4(position, 1.0f);
+    // Basic particle animation - you can enhance this based on your needs
+    vec3 animated_pos = position;
+    
+    // Optional: Add some movement or rotation based on time
+    // animated_pos.y += sin(time + position.x) * 0.1;
+    
+    gl_Position = vec4(animated_pos, 1.0f);
+    
+    l_fragpos = vec3(model * vec4(animated_pos, 1.0));
+    l_normal = mat3(transpose(inverse(model))) * normal;
+    
+    tex_coords = texCoords;
+    frag_color = colors;
 }
 """  # type: str
 
@@ -170,9 +197,10 @@ out vec4 FragColor;
 
 in vec2 tex_coords;
 in vec3 frag_color;
-
 in vec3 l_fragpos;
 in vec3 l_normal;
+in vec3 view_pos;
+in vec4 frag_pos_light_space;
 
 uniform vec3 light_pos[100]; // assume 100 lights max.
 uniform vec3 light_color[100];
@@ -188,10 +216,15 @@ uniform float opacity;
 uniform float far_plane;
 uniform int lit;
 
+// Material properties for enhanced shading (with defaults to maintain compatibility)
+uniform float metallic;
+uniform float roughness;
+uniform float ao;
+
 uniform sampler2D tex_unit;
 uniform samplerCube depthMap;
 
-// array of offset direction for sampling
+// Improved sampling pattern for shadows
 vec3 gridSamplingDisk[20] = vec3[]
 (
    vec3(1, 1,  1), vec3( 1, -1,  1), vec3(-1, -1,  1), vec3(-1, 1,  1),
@@ -201,17 +234,25 @@ vec3 gridSamplingDisk[20] = vec3[]
    vec3(0, 1,  1), vec3( 0, -1,  1), vec3( 0, -1, -1), vec3( 0, 1, -1)
 );
 
+// Enhanced shadow calculation with better bias
 float ShadowCalculation(vec3 fragPos)
 {
+    if (LIGHT_COUNT == 0) return 0.0;
+    
     vec3 fragToLight = fragPos - light_pos[0];
     float currentDepth = length(fragToLight);
     float shadow = 0.0;
-    float bias = 0.15;
+    
+    // Improved bias calculation
+    vec3 normal = normalize(l_normal);
+    vec3 lightDir = normalize(light_pos[0] - fragPos);
+    float bias = max(0.05 * (1.0 - dot(normal, lightDir)), 0.005);
 
     float viewDistance = length(camera_pos - fragPos);
     float diskRadius = (1.0 + (viewDistance / far_plane)) / 25.0;
 
-    for(int i = 0; i < samples; ++i)
+    int valid_samples = min(samples, 20);
+    for(int i = 0; i < valid_samples; ++i)
     {
         float closestDepth = texture(depthMap, fragToLight +
                                      gridSamplingDisk[i] * diskRadius).r;
@@ -220,50 +261,87 @@ float ShadowCalculation(vec3 fragPos)
              shadow += 1.0;
     }
 
-    return shadow / float(samples);
+    return shadow / float(valid_samples);
 }
 
 void main()
 {
     vec3 color;
     float alpha;
+    
+    // Material color determination
     if (material_mode == 0 || material_mode == 2) {
         color = object_color;
         alpha = opacity;
     }
-    if (material_mode == 1 || material_mode == 3) {
-        color = texture(tex_unit, tex_coords).rgb;
-        alpha = texture(tex_unit, tex_coords).a;
+    else if (material_mode == 1 || material_mode == 3) {
+        vec4 tex_sample = texture(tex_unit, tex_coords);
+        color = tex_sample.rgb;
+        alpha = tex_sample.a * opacity;
     }
-    if (material_mode == 4) {
+    else if (material_mode == 4) {
         color = frag_color;
         alpha = opacity;
     }
-
-    if (lit == 0) {
-        FragColor = vec4(color, alpha);
-    } else {
-        vec3 norm = normalize(l_normal);
-        for (int i = 0; i < LIGHT_COUNT; i++) {
-            vec3 ambient = 0.3 * light_color[i];
-
-            // diffuse
-            vec3 norm = normalize(l_normal);
-            vec3 lightDir = normalize(light_pos[i] - l_fragpos);
-            float diff = max(dot(norm, lightDir), 0.0);
-            vec3 diffuse = diff * light_color[i];
-            float spec = 0.0;
-            vec3 halfwayDir = normalize(lightDir + camera_pos);
-            spec = pow(max(dot(norm, halfwayDir), 0.0), 64.0);
-
-            vec3 specular = spec * light_color[i];
-            float shadow = shadow_enabled ? ShadowCalculation(l_fragpos) : 0.0;
-            vec3 lighting = (ambient + (1.0 - shadow) *
-                             (diffuse + specular)) * color;
-
-            FragColor = vec4(lighting, alpha);
-        }
+    else {
+        color = object_color;
+        alpha = opacity;
     }
+
+    if (lit == 0 || LIGHT_COUNT == 0) {
+        // Unlit or no lights - just show the color
+        FragColor = vec4(color, alpha);
+        return;
+    }
+    
+    // Lighting calculations
+    vec3 norm = normalize(l_normal);
+    
+    // Check for invalid normals
+    if (length(l_normal) < 0.1) {
+        FragColor = vec4(color * 0.5, alpha); // Fallback for invalid normals
+        return;
+    }
+    
+    vec3 viewDir = normalize(camera_pos - l_fragpos);
+    vec3 result = vec3(0.0);
+    
+    for (int i = 0; i < min(LIGHT_COUNT, 100); i++) {
+        vec3 lightDir = normalize(light_pos[i] - l_fragpos);
+        float distance = length(light_pos[i] - l_fragpos);
+        
+        // Enhanced attenuation
+        float attenuation = 1.0 / (1.0 + 0.045 * distance + 0.0075 * distance * distance);
+        
+        // Ambient
+        vec3 ambient = 0.15 * color * ao;
+        
+        // Diffuse
+        float diff = max(dot(norm, lightDir), 0.0);
+        vec3 diffuse = diff * light_color[i] * color;
+        
+        // Specular (Blinn-Phong)
+        vec3 halfwayDir = normalize(lightDir + viewDir);
+        float spec_strength = mix(0.5, 1.0, metallic); // Use metallic to control specular strength
+        float shininess = mix(32.0, 128.0, 1.0 - roughness); // Use roughness to control shininess
+        float spec = pow(max(dot(norm, halfwayDir), 0.0), shininess);
+        vec3 specular = spec_strength * spec * light_color[i];
+        
+        // Shadow calculation (only for first light to keep performance)
+        float shadow = (i == 0 && shadow_enabled) ? ShadowCalculation(l_fragpos) : 0.0;
+        
+        // Combine results
+        vec3 lighting = ambient + (1.0 - shadow) * (diffuse + specular);
+        result += lighting * attenuation;
+    }
+    
+    // Simple tone mapping
+    result = result / (result + vec3(1.0));
+    
+    // Gamma correction
+    result = pow(result, vec3(1.0/2.2));
+    
+    FragColor = vec4(result, alpha);
 }"""  # type: str
 
 
@@ -276,29 +354,42 @@ layout ( location = 3 ) in vec3 colors; // optional
 
 out vec2 tex_coords;
 out vec3 frag_color;
+out vec3 l_fragpos;
+out vec3 l_normal;
+out vec3 view_pos;
+out vec4 frag_pos_light_space;
 
 uniform mat4 model;
 uniform mat4 view;
 uniform mat4 projection;
+uniform mat4 light_space_matrix;
 uniform int view_mode;
 
-out vec3 l_fragpos;
-out vec3 l_normal;
-
 void main()
 {
-    l_fragpos = vec3(model * vec4(position, 1.0));
-    l_normal = mat3(transpose(inverse(model))) * normal;
+    vec4 world_pos = model * vec4(position, 1.0);
+    l_fragpos = world_pos.xyz;
+    
+    // Transform normal to world space with proper handling of non-uniform scaling
+    mat3 normal_matrix = mat3(transpose(inverse(model)));
+    l_normal = normalize(normal_matrix * normal);
+    
+    // Calculate view position for PBR calculations
+    view_pos = (view * world_pos).xyz;
+    
+    // Calculate light space position for shadow mapping
+    frag_pos_light_space = light_space_matrix * world_pos;
 
     if (view_mode == 0) {
-        gl_Position = projection * view * model * vec4(position, 1.0f);
+        gl_Position = projection * view * world_pos;
     }
     if (view_mode == 1) {
-        gl_Position = (projection * (vec4(position, 1.0f)
-                        + vec4(model[3].xyz, 0.0f)));
+        gl_Position = projection * (vec4(position, 1.0f) + vec4(model[3].xyz, 0.0f));
     }
 
-    gl_PointSize = 5.0;
+    // Enhanced point size with distance attenuation
+    float distance_to_camera = length(view_pos);
+    gl_PointSize = max(1.0, 10.0 / (1.0 + 0.1 * distance_to_camera));
 
     tex_coords = texCoords;
     frag_color = colors;
@@ -323,12 +414,30 @@ background_fragment_shader = """
 #version 330 core
 uniform vec4 top_color;
 uniform vec4 bot_color;
+uniform float time;
+uniform vec2 resolution;
 in vec2 v_uv;
 
 out vec4 frag_color;
+
 void main()
 {
-  frag_color = top_color * (1 - v_uv.y) + bot_color * v_uv.y;
+    // Enhanced gradient with subtle animation
+    float gradient_factor = smoothstep(0.0, 1.0, v_uv.y);
+    
+    // Add subtle color variation over time (optional)
+    float time_variation = sin(time * 0.1) * 0.02 + 1.0;
+    
+    vec4 mixed_color = mix(bot_color, top_color, gradient_factor) * time_variation;
+    
+    // Add subtle noise for more natural look
+    float noise = fract(sin(dot(v_uv, vec2(12.9898, 78.233))) * 43758.5453) * 0.02 - 0.01;
+    mixed_color.rgb += noise;
+    
+    // Ensure colors stay in valid range
+    mixed_color = clamp(mixed_color, 0.0, 1.0);
+    
+    frag_color = mixed_color;
 }
 """  # type: str
 
@@ -362,7 +471,38 @@ class Shader:
         self.vertex_shader_source = vertex
         self.geometry_shader_source = geometry
 
-        self.variables: List[str] = [] if variables is None else variables
+        # Enhanced default variables for improved visual quality
+        default_vars = [
+            "model",
+            "view",
+            "projection",
+            "light_space_matrix",
+            "object_color",
+            "opacity",
+            "metallic",
+            "roughness",
+            "ao",
+            "camera_pos",
+            "light_pos",
+            "light_color",
+            "LIGHT_COUNT",
+            "material_mode",
+            "lit",
+            "shadow_enabled",
+            "samples",
+            "far_plane",
+            "time",
+            "resolution",
+            "top_color",
+            "bot_color",
+            "particle_size",
+            "tex_unit",
+            "depthMap",
+        ]
+
+        self.variables: List[str] = (
+            default_vars if variables is None else variables + default_vars
+        )
         self._stack: Dict[str, int] = {}  # Variable stack.
         self._mode: int = self.NO_LIGHT_COLOR  # Lightless color material
         self._depth_shader = False
@@ -473,7 +613,8 @@ class Shader:
         location = self.get_location(variable)
         if location == -1:
             return False
-        glUniform3fv(location, count, value[:3])
+        # Send count * 3 values (3 components per vector)
+        glUniform3fv(location, count, value[: count * 3])
         return True
 
     def set_vector3_np(self, variable: str, value: np.ndarray) -> bool:
@@ -538,3 +679,51 @@ class Shader:
 
         glUniform1f(location, ctypes.c_float(value))
         return True
+
+    def set_material_properties(
+        self, metallic: float = 0.0, roughness: float = 0.5, ao: float = 1.0
+    ) -> None:
+        """Set PBR material properties
+
+        Keyword arguments:
+        metallic -- Metallic factor (0.0 = dielectric, 1.0 = metallic)
+        roughness -- Surface roughness (0.0 = mirror, 1.0 = completely rough)
+        ao -- Ambient occlusion factor (0.0 = fully occluded, 1.0 = no occlusion)
+        """
+        # Ensure the uniforms exist and set safe defaults
+        if self.get_location("metallic") == -1:
+            # If uniform doesn't exist, shader might not use PBR
+            return
+
+        self.set_float("metallic", metallic)
+        self.set_float("roughness", max(0.04, roughness))  # Prevent roughness of 0
+        self.set_float("ao", ao)
+
+    def set_default_material_properties(self) -> None:
+        """Set default material properties for backward compatibility"""
+        self.set_material_properties(0.0, 0.5, 1.0)
+
+    def set_time(self, time: float) -> bool:
+        """Set time uniform for animated shaders
+
+        Keyword arguments:
+        time -- Time value in seconds
+        """
+        return self.set_float("time", time)
+
+    def set_resolution(self, width: float, height: float) -> bool:
+        """Set screen resolution for shaders that need it
+
+        Keyword arguments:
+        width -- Screen width
+        height -- Screen height
+        """
+        location = self.get_location("resolution")
+        if location == -1:
+            return False
+
+        import numpy as np
+
+        resolution = np.array([width, height], dtype=np.float32)
+        glUniform2fv(location, 1, resolution)
+        return True
